@@ -11,6 +11,7 @@ __license__ = "ZPL"
 import csv
 import os.path
 import re
+import fullname as modfullname
 
 __MY__PATH__ = os.path.dirname(os.path.abspath(__file__))
 glEntities = dict([p for p in csv.reader(open(os.path.join(__MY__PATH__,"entities.csv"),"rU")) ]) 
@@ -60,9 +61,14 @@ def tofilename(name):
     for s in illegal : n = n.replace(s,'')
     return n
 
-def parse_fullname(fullname):
-    """ Parses a fullname into pieces: firstname, lastname, middlename, suffix.
-        Returns dict """
+def parse_fullname(fullname, schema="lfms"):
+    """ Parse fullname using a schema.
+        l - lastname
+        m - firstname
+        f - firstname
+        s - suffix
+        into pieces: firstname, lastname, middlename, suffix.
+        Returns dict     """
     job = dict()
     job['firstname'] = job['lastname'] = job['middlename'] = job['suffix'] = ''
     if ", " in fullname:
@@ -84,13 +90,18 @@ def parse_fullname(fullname):
             allnames = allnames[:-1]
     if len(allnames) == 1:
         job['lastname'] = allnames[0]
-    elif len(allnames) == 2:
-         job['lastname'] = allnames[0]
-         job['firstname'] = allnames[1]
-    elif len(allnames) > 2:
-         job['lastname'] = allnames[0]
-         job['firstname'] = allnames[1]
-         job['middlename'] = " ".join(allnames[2:])
+    else:
+        if schema == "lfms":
+            parser = modfullname.parse_lfms
+        elif schema == "lmfs":
+            parser = modfullname.parse_lmfs
+        elif schema == "fmls":
+            parser = modfullname.parse_fmls
+        elif schema == "flms":
+            parser = modfullname.parse_flms
+        else:
+            raise ValueError("Unknown fullname schema %s" % schema)
+    job.update(parser(allnames))
     # postprocess middlename
     if "; " in job['middlename']:
         job['middlename'], job['suffix'] = job['middlename'].split("; ",1)
@@ -98,6 +109,7 @@ def parse_fullname(fullname):
         for c in (',','.',';'):
             job[f] = job[f].replace(c," ")
         while "  " in job[f]: job[f] = job[f].replace("  "," ")
+
     return job
 
 def parse_fulladdress(fulladdress):
@@ -110,14 +122,13 @@ def parse_fulladdress(fulladdress):
         info["address"] = fulladdress
         return info
     rest, info["zip"] = fulladdress.rsplit(" ", 1)
-    if not all(map(lambda x:x in "0123456789-", info["zip"])): 
+    if len(info["zip"]) == 2: 
         info['state'] = info['zip']
         info['zip'] = ''
     if ", " not in  rest:
         info["address"] = rest
         return info
-    if not info["state"]:
-        rest, info["state"] = rest.rsplit(", ", 1)
+    rest, info["state"] = rest.rsplit(", ", 1)
     if ", " not in  rest:
         info["address"] = rest
         return info
@@ -136,7 +147,6 @@ def remove_aka(fullname):
             fullname = fullname.split(aka,1)[0]
     return fullname
 
-
 def is_person(fullname):
     """ Checks whether a name given is person's name """
     return not (any(map(lambda e:fullname.upper().strip().endswith(e), 
